open Interpreter
open Typer
open Ast
open Instr

let run1 () =
  type_and_eval (Eunary(Oneg, Ebinary(Oplus, Ebyte(126), Ebyte(2))));
  type_and_eval (Ebincomp(Oequal, Eint(2), Efloat(3.0)));
;;

let run2 () =
  let print_char c = Prim.output_char Codegen.stdout c in
    Codegen.initialise_code ();
    String.iter print_char "Hello world !!!\n";
    Prim.flush Codegen.stdout;
    Prim.input_char Codegen.stdin;
    Codegen.finalise_code ();
    Linker.export "a.out";
;;

let type_and_gen_expr ast =
  let typed_ast =
    try Typer.type_exp Typer.empty_env ast
    with Type_error(_) as te -> Printer.print_type_error stderr te
  in
    Codegen.initialise_code ();
    Codegen.gen_expr typed_ast;
    begin match fst typed_ast with
      | `float -> Codegen.gen_string_of_float ();
      | `int | `short | `byte -> Codegen.gen_string_of_int ();
      | `boolean -> Codegen.gen_string_of_bool ();
      | `string -> ()
      | _ -> failwith "faut bosser benoît !";
    end;
    Codegen.gen_print_string ();
    Prim.output_char Codegen.stdout '\n';
    Codegen.finalise_code ();
    Linker.export "a.out";
;;

let type_and_gen_instr ast =
  let typed_ast =
    try Typer.type_instr Typer.empty_env ast
    with Type_error(_) as te -> Printer.print_type_error stderr te
  in
    Codegen.initialise_code ();
    Codegen.gen_instr typed_ast;
    Codegen.finalise_code ();
    Linker.export "a.out";
;;

let run3 () =
  type_and_gen_expr (Eunary(Oneg, Ebinary(Oplus, Ebyte(126), Ebyte(2))));
;;

let run4 () =
  type_and_gen_expr (Efloat (3.14));
;;

let run5 () =
  let lexbuf = Lexing.from_string "\
println(\"Hello world !!!\");
println(\"Hello \" + 42 + \" \" + 3.14);
//println(\"Hello\" == \"Hello\");
/*
while(true){
  println(\"x \" + 816 + \" y\");
}
*/
" in
  let ast = Iblock (Parser.block_statements_opt Lexer.main lexbuf) in
    Printer.print_instruction 2 stdout ast;
    type_and_gen_instr ast;
;;
